// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "SimpleSpinningCube.hpp"
#include "display/mesh/CubeMesh.hpp"
#include "component/SimpleComponent.hpp"

using namespace Dawn;

struct SimpleSpinningCube Dawn::createSimpleSpinningCube(Scene &s) {
  struct SimpleSpinningCube cube;

  // Create the scene item.
  cube.item = s.createSceneItem();

  // Create a simple cube mesh.
  cube.mesh = std::make_shared<Mesh>();
  cube.mesh->createBuffers(CUBE_VERTICE_COUNT, CUBE_INDICE_COUNT);
  CubeMesh::buffer(cube.mesh, glm::vec3(-1, -1, -1), glm::vec3(2, 2, 2), 0, 0);

  // Add a renderer to the scene item.
  cube.meshRenderer = cube.item->addComponent<MeshRenderer>();
  cube.meshRenderer->mesh = cube.mesh;

  // Add a material to the scene item.
  cube.material = cube.item->addComponent<SimpleTexturedMaterial>();
  cube.material->setColor(COLOR_MAGENTA);

  // Add a simple event listener component to the scene item.
  addSimpleComponent(cube.item, [](auto &cmp, auto &events) {
    // Note that add component cannot receive a self reference, so we must
    // capture the component by reference instead.
    events.push_back(cmp.getScene()->onUnpausedUpdate.listen([&](
      float_t delta
    ) {
      // Since we captured within the lambda, we can access the component
      // directly.
      auto item = cmp.getItem();
      item->setLocalRotation(
        item->getLocalRotation() *
        glm::quat(glm::vec3(1, 1, 0) * delta)
      );
    }));
  });

  return cube;
}