// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "dawnlibs.hpp"

namespace Dawn {
  class Easing final {
    public:
      /**
       * Linear easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t linear(float_t t);

      /**
       * Quadratic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInQuad(float_t t);

      /**
       * Quadratic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeOutQuad(float_t t);

      /**
       * Quadratic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInOutQuad(float_t t);

      /**
       * Cubic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInCubic(float_t t);
      
      /**
       * Cubic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeOutCubic(float_t t);

      /**
       * Cubic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInOutCubic(float_t t);

      /**
       * Quartic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInQuart(float_t t);

      /**
       * Quartic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeOutQuart(float_t t);

      /**
       * Quartic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInOutQuart(float_t t);

      /**
       * Quintic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInQuint(float_t t);

      /**
       * Quintic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeOutQuint(float_t t);

      /**
       * Quintic easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInOutQuint(float_t t);

      /**
       * Sine easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInSine(float_t t);

      /**
       * Sine easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeOutSine(float_t t);

      /**
       * Sine easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInOutSine(float_t t);

      /**
       * Exponential easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInExpo(float_t t);

      /**
       * Exponential easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeOutExpo(float_t t);

      /**
       * Exponential easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInOutExpo(float_t t);

      /**
       * Circular easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInCirc(float_t t);

      /**
       * Circular easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeOutCirc(float_t t);

      /**
       * Circular easing function.
       * 
       * @param t Time value between 0 and 1.
       * @return Eased value.
       */
      static float_t easeInOutCirc(float_t t);
  };
}