// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "PlayerPrefab.hpp"
#include "display/mesh/CubeMesh.hpp"

using namespace Dawn;

struct PlayerPrefab Dawn::createPlayerPrefab(
  const std::shared_ptr<Map> map
) {
  struct PlayerPrefab player;
  player.item = map->getScene()->createSceneItem();
  player.item->setParent(map->getItem());

  player.player = player.item->addComponent<Player>();
  player.player->map = map;
  player.player->id = EntityID::Player;

  player.mesh = std::make_shared<Mesh>();
  player.mesh->createBuffers(CUBE_VERTICE_COUNT, CUBE_INDICE_COUNT);
  CubeMesh::buffer(
    player.mesh,
    glm::vec3(-0.5f, -0.5f, -0.5f),
    glm::vec3(1, 1, 1),
    0, 0
  );

  player.meshRenderer = player.item->addComponent<MeshRenderer>();
  player.meshRenderer->mesh = player.mesh;

  player.material = player.item->addComponent<SimpleTexturedMaterial>();
  player.material->setColor(COLOR_MAGENTA);

  return player;
}