// Copyright (c) 2024 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#include "TestEntityPrefab.hpp"
#include "display/mesh/CubeMesh.hpp"

using namespace Dawn;

struct TestEntityPrefab Dawn::createTestEntityPrefab(
  const std::shared_ptr<Map> map
) {
  struct TestEntityPrefab entity;
  entity.item = map->getScene()->createSceneItem();
  entity.item->setParent(map->getItem());

  entity.entity = entity.item->addComponent<Entity>();
  entity.entity->map = map;
  entity.entity->id = EntityID::TestEntity;

  entity.mesh = std::make_shared<Mesh>();
  entity.mesh->createBuffers(CUBE_VERTICE_COUNT, CUBE_INDICE_COUNT);
  CubeMesh::buffer(
    entity.mesh,
    glm::vec3(-0.5f, -0.5f, -0.5f),
    glm::vec3(1, 1, 1),
    0, 0
  );

  entity.meshRenderer = entity.item->addComponent<MeshRenderer>();
  entity.meshRenderer->mesh = entity.mesh;

  entity.material = entity.item->addComponent<SimpleTexturedMaterial>();
  entity.material->setColor(COLOR_GREEN);

  return entity;
}